% //This software is licensed under the BSD 3 Clause license: http://opensource.org/licenses/BSD-3-Clause 
% 
% 
% //Copyright (c) 2013, University of Oxford
% //All rights reserved.
% 
% //Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% //Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% //Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% //Neither the name of the University of Oxford nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
% //THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%
%   The method implemented in this file has been patented by their original
%   authors. Commercial use of this code is thus strongly not
%   recocomended.
%
% //Authors: 	Gari D Clifford - 
% //            Roberta Colloca -
% //			Julien Oster	-

% Modified Joachim Behar 23-03-2017

function [isAF,CosEn] = afMoormanDetector(data,W,th)

%    afDetector takes the RR intervals series 'data' and computes the value of CosEn
%    for each W-beat segment

%    The input vector 'data' is a column vector and it should be at least
%    W-samples long: cosEn value, for each W-beat segment of the vector is 
%    computed and stored in the column vector CosEn.

%    Values of CosEn are compare with the AF threshold and the diagnosis
%    for each W-beat segment is stored in the column vector AF

%    Input
%    data: RR intervals series (column vector)
%    W: Window size 
%    th: threshold for AF detection
%
%    Output
%    isAF: vector column containing 1 if AF and 0 otherwise 
%          (length is equal to the length of data divided by W) 
 


L=length(data);
CosEn=[];
SAMPE=[];
MEANRR=[];
MINRR=[];
MEDFr=[];
j=1;
w=W;   %window length
i=w;      
%th= th_i;  %CosEn threshold: diagnosis of AF is assigned for values >th

while i<=L
    [cosEn,sentropy,mRR,minRR,medFreq]=comp_cosEn(data(i-w+1:i,1));
    CosEn(j,1) = cosEn;
    SAMPE(j,1)= sentropy;
    MEANRR(j,1)=mRR;
    MINRR(j,1)=minRR;
    MEDFr(j,1)=medFreq;
    j=j+1;
    i=i+w;
end

%MOORMANfeatures=[CosEn,SAMPE,MEANRR,MINRR,MEDFr];


%detection of AF by comparing Cosen witha  threshold
isAF = zeros(size(CosEn));
isAF(CosEn>th)=1;

end
