function [ F1, IndMatch, meanDist ] = bsqi_matlab( refqrs, testqrs,thres,fs)
%BSQI_MATLAB Calculate bSQI of two inputs
%   Detailed explanation goes here
%
% Copyright (C) 2017 Joachim A. Behar
% Bio-electric and Bio-energetic system Group, Technion-IIT, Israel
% joachim.a.behar@gmail.com
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 2 of the License, or (at your
% option) any later version.
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.

% == managing inputs
if nargin<3; thres=0.05; end;
if nargin<4; fs=250; end;
thres = thres * fs;

if ~isempty(refqrs) && ~isempty(testqrs)
    NB_REF  = length(refqrs);
    NB_TEST = length(testqrs);
    % == core function
    [IndMatch,Dist] = dsearchn(refqrs,testqrs);         % closest ref for each point in test qrs
    IndMatchInWindow = IndMatch(Dist<thres);         % keep only the ones within a certain window
    NB_MATCH_UNIQUE = length(unique(IndMatchInWindow)); % how many unique matching
    TP = NB_MATCH_UNIQUE;                               % number of identified ref QRS
    FN = NB_REF-TP;                                     % number of missed ref QRS
    FP = NB_TEST-TP;                                    % how many extra detection?
    Se  = TP/(TP+FN);
    PPV = TP/(FP+TP);
    F1 = 2*Se*PPV/(Se+PPV);                             % accuracy measure
    
    
    %test JOs get mean distance between ref and testqrs
    [~ ,ind_plop] = unique(IndMatchInWindow);
    Dist_thres = find(Dist<thres);
    meanDist = mean(Dist(Dist_thres(ind_plop)))./fs;
    
else
    F1 = 0;
    IndMatch = [];
    meanDist = fs;
end
end
