function classifyResult = challenge(recordName)
%
% Written by: Chengyu Liu and Qiao Li January 20 2017
%             chengyu.liu@emory.edu  qiao.li@emory.edu
%
% Last modified by: joachim behar

addpath(genpath('hrv_Aviv_toolbox/'));
addpath(genpath('ecg-kit-0.1.0/'));
%addpath(genpath('libsvm-3.20/'));

classifyResult = 'N'; % default output normal rhythm
USE_COMPUTED_QRS = 0;

if ~exist('opt')
    %% == MAIN PARAMETERS
%     opt.THRES_SQI = 0.8; % SQI threshold                                    [sqi step]
%     opt.WIND_SQI = 0.05; % agreement window for the SQI                     [sqi step]
%     opt.REG_WIN = 1; % pace at which to compute sqi - every second          [sqi step]
%     opt.SIZE_WIND = 5; % window size on which the sqi is computed           [sqi step]
%     opt.LG_MED = 1; % median if we want some smoothing of the sqi           [sqi step]
%     opt.THERS_TAC = 100; % threshold for tachycardia                        [other rythms detection step]
%     opt.THERS_BRA = 50; % threshold brady                                   [other rythms detection step]
%     opt.THRES_AF = -1.3; % threshold of AF detector                         [AF detection step]
%     opt.MIN_AF_RR = 5; % minimum number of RR to perform AF detection       [AF detection step]
%     %opt.WIND_NB_SAMP = 2; % number of samples in averaging window          [ectopic beat rejection step]
%     opt.WIND_TOL = 20; % window tolerance in %                              [ectopic beat rejection step]
%     opt.MIN_NB_FILT = 10; % minimum number of beats                         [ectopic beat rejection step]
%     opt.JQRS_THRESH = 0.6; % jqrs detector threshold                        [qrs detection step]
%     opt.JQRS_WINDOW = 10; % jqrs window size                                [qrs detection step]
%     opt.JQRS_REFRAC = 0.15; % jqrs refractory period                        [qrs detection step]
%     opt.JQRS_INTWIN_SZ = 7; % integration window                            [qrs detection step]
%     opt.MIN_SUB_SEG_TIME = 10; % minimal time of the subsegment that is selected for doing the analysis [sqi step]
%     opt.False_Outlier_Thres = -0.1; % threshold for outlier detection           [ectopic beat rejection step]
%     opt.WIND_NB_SAMP = 10; % window size for median filter in beat rejection    [ectopic beat rejection step]
%     opt.sqi_outliers = 0.99; % threshold sqi value at point of outlier          [ectopic beat rejection step]
%     opt.bimod_alt = 0.85; % percentage of time series alternating sign                          [bimodal step]
%     opt.bimod_delta_perc = 0.50; % percentage segment with minimum opt.bimod_delta variation    [bimodal step]
%     opt.bimod_delta = 15; % percentage HR variation between modes                               [bimodal step]

opt.THRES_SQI = 0.9229;
opt.WIND_SQI = 0.0796;
opt.REG_WIN = 1;
opt.SIZE_WIND = 3;
opt.LG_MED = 1;
opt.THERS_TAC = 101;
opt.THERS_BRA = 49;
opt.THRES_AF = -1.2318;
opt.MIN_AF_RR = 5;
opt.WIND_TOL = 27.987;
opt.MIN_NB_FILT = 10;
opt.JQRS_THRESH = 0.6;
opt.JQRS_REFRAC = 0.15;
opt.JQRS_INTWIN_SZ = 7;
opt.MIN_SUB_SEG_TIME = 7;
opt.False_Outlier_Thres = -0.1027;
opt.WIND_NB_SAMP = 12;
opt.sqi_outliers = 0.8874;
opt.bimod_alt = 0.91;
opt.bimod_delta_perc = 0.3916;
opt.bimod_delta = 18.9001;

end

jqrs = []; epltd = []; anns_gqrs = []; anns_wqrs = [];

[~,ecg,fs,siginfo]=rdmat(recordName);

NB_PT = length(ecg);

siginfo.fmt = '16'; % TEST !!!

%% [qrs detection step]
%%
script_qrs_detection_all;


%% [prefilter qrs time series]
%%
[jqrs,epltd,anns_gqrs,anns_wqrs] = prefilter_qrs(ecg,jqrs,epltd,anns_gqrs,anns_wqrs,fs,NB_PT);


%% [sqi step]
%%
%test JOs
%epltd =[];
[F1,sqi,tsqi,QRS,start_seg,end_seg,meanSQI,medianSQI,quartileSQI,dist_jqrs_gqrs] = sqi_step(jqrs,epltd,anns_gqrs,fs,NB_PT,opt);

%% [ectopic beat detection step]
%%
%[lp_outliers,rr_lp] = detect_outlier(QRS,fs,opt,sqi);
[~,lp_outliers,rr_lp,newQRS] = detect_outlier(QRS,fs,opt,sqi); % FIXME
%test JOs
QRS = newQRS;
%% [binary mode test]
%%
hr = 60./(diff(QRS)/fs);

% = test if HR change sign every beat but it is not checking that the HR
% has two modes so to say.
diff_hr = diff(hr);
diff_hr_1 = diff_hr(1:2:end);
diff_hr_2 = diff_hr(2:2:end);

if length(diff_hr_2)<length(diff_hr_1)
    diff_hr_1 = diff_hr_1(1:end-1);
end

sg = sign(diff_hr_1).*sign(diff_hr_2);
amp_var = 100*abs(diff_hr_1/median(hr));

if sum(sg==-1)>round(length(sg)*opt.bimod_alt) && length(sg)>6 ...
        && sum(amp_var>opt.bimod_delta)>ceil(length(sg)*opt.bimod_delta_perc)
    binar = 1;
else
    binar = 0;
end


%% Now decide what label each segment has
if length(QRS)<6
    disp('==================> Less than 6 beats!')
end

if length(QRS)<6 || F1<=opt.THRES_SQI
    % [sqi detection step]
    classifyResult = '~';
    CosEn = NaN;
    AFE = NaN;
    OrC = NaN;
    IrE = NaN;
    PACe = NaN;
    RR = diff(QRS')/fs;
    
    out = 0;
    if 60/median(RR)>opt.THERS_TAC; out=1; end
    if 60/median(RR)<opt.THERS_BRA; out=1; end
else
    RR = diff(QRS')/fs;
    
    % [AF detection step]
    if floor(sum(RR))>opt.MIN_AF_RR
        [isAF,CosEn] = afMoormanDetector(RR,length(RR),opt.THRES_AF);
        [ AFE,OrC,IrE,PACe ]=comput_AFEv(RR);
        if (AFE/length(RR)<0.3)
            isAF=0;
        end
        
    else
        isAF = 0;
        CosEn = NaN;
        AFE = NaN;
        OrC = NaN;
        IrE = NaN;
        PACe = NaN;
    end
    
    
    out = 0; % FIXME
    if 60/median(rr_lp)>opt.THERS_TAC; out=1; end % FIXME
    if 60/median(rr_lp)<opt.THERS_BRA; out=1; end % FIXME

end


%% feature vector
min_rr = min(rr_lp);
max_rr = max(rr_lp);
median_rr = median(rr_lp);
nb_outliers = length(lp_outliers);
[hrv_frag,hrv_td] = hrv_features(RR);
length_clean = end_seg-start_seg+1;

%%
%% BEGIN NEW
% == ECG segmentation
try
    ecg_select = ecg(round(start_seg*fs)+1:round(end_seg*fs));
    qrs_select = jqrs(jqrs>start_seg*fs & jqrs<end_seg*fs)-ceil(start_seg*fs);
    
    heasig.nsig = 1;
    heasig.freq = 300;
    
    ecg_select_res = resample(ecg_select,heasig.freq,fs);
    qrs_select_res = round(qrs_select*heasig.freq/fs);
    
    heasig.nsamp = length(ecg_select_res);
    
    [position,~,~] = wavedet_3D(ecg_select_res, qrs_select_res, heasig, []);
catch
    position = [];
end
% == baseline wander removal
[b_bas,a_bas] = butter(2,0.7/(heasig.freq/2),'high');
bp_ecg_select_res = filtfilt(b_bas,a_bas,ecg_select_res);
bp_ecg_select_res = filtfilt(b_bas,a_bas,bp_ecg_select_res);

% ====> NEW Looking if we have a continuous HR above or below tachy/brady
% threshold and if yes take it otherwise take the median HR over the whole
% signal
[~,med_tb] = brady_tachy_v2(QRS,fs,opt.THERS_BRA,opt.THERS_TAC);

% == f-wave detection for P-wave
[ratio,max_freq] = f_wave_detection(ecg_select,qrs_select,fs);

% == QRS length
[~,~,~,~,R_amp_m,R_amp_std] = compute_qrs_size(ecg_select,qrs_select,fs,0);

% ====> UPDATED Fix to Q and S fiducials
ind_Q_vec = round(position.QRSon*(fs/heasig.freq))';
ind_S_vec = round(position.QRSoff*(fs/heasig.freq))';
QS = nanmedian(ind_S_vec - ind_Q_vec)/fs;
QS_std = nanstd(ind_S_vec - ind_Q_vec)/fs;


% == getting all the morphological intervals
medRR = median(diff(qrs_select)/fs);
if ~isempty(position)
    medQT =  nanmedian((position.Toff'/heasig.freq-ind_Q_vec/fs)); % important use nanmedian
    [medQT_b, medQT_fre, medQT_fra, medQT_hod] = corrected_qt(medQT, medRR);
    medP = nanmedian((position.Poff - position.Pon)/heasig.freq); % median P-wave length
    stdP = nanstd((position.Poff - position.Pon)/heasig.freq); % std of P-wave length
    medT = nanmedian((position.Toff - position.Ton)/heasig.freq); % median T-wave length
    stdT = nanstd((position.Toff - position.Ton)/heasig.freq); % std of T-wave length
    medPR = nanmedian(ind_Q_vec'/fs-position.Pon/heasig.freq); % median PR interval length
    stdPR = nanstd(ind_Q_vec'/fs - position.Pon/heasig.freq); % std of PR interval length
    medPRseg = nanmedian(ind_Q_vec'/fs-position.Poff/heasig.freq); % median PR segment length
    indTamp = find(~isnan(position.T) & ~isnan(position.Toff)); % check beats for which we have both T amplitude and Toff fiducials
    medTamp = nanmedian(bp_ecg_select_res(position.T(indTamp)) - bp_ecg_select_res(position.Toff(indTamp))); % median T-wave amplitude (taken as max - Tend)
    stdTamp = nanstd(bp_ecg_select_res(position.T(indTamp)) - bp_ecg_select_res(position.Toff(indTamp))); % std T-wave amplitude
    medTtype = median(position.Ttipo); % median T-wave type (1:5 corresponding respectively to normal, inverted, upwards only, downwards only, biphasc pos-neg, biphasc neg-pos)

    nbpwaves = length(~isnan(position.Poff - position.Pon))*medRR/heasig.nsamp; % NEW UPDATED number of P-wave detected by unit of time normalised by the heart rate. Idea is that less P-wave will be detected in AF cases
    indPamp = find(~isnan(position.P) & ~isnan(position.Poff)); % beats for which we have a P amplitude and Poff fiducials
    medPamp = nanmedian(bp_ecg_select_res(position.P(indPamp)) - bp_ecg_select_res(position.Poff(indPamp))); % NEW median P-wave amplitude computed from peak P-wave to Poff
    stdPamp = nanstd(bp_ecg_select_res(position.P(indPamp)) - bp_ecg_select_res(position.Poff(indPamp))); % NEW std of P-wave amplitude
    
    indSTamp = find(~isnan(position.QRSoff) & ~isnan(position.Ton));
    medST = nanmedian((position.Ton(indSTamp)-position.QRSoff(indSTamp))/heasig.freq); % NEW: ST segment length. S point starting at QRSoff and finishing at Ton
    
    % == NEW: ST segment elevation/depletion
    indPRbas = find(~isnan(position.Poff) & ~isnan(position.QRSon) & ~isnan(position.QRSoff) & ~isnan(position.Ton)); % only select cycles for which we get the Poff AND QRSon AND QRSoff AND Ton
    nbPRbas = length(indPRbas);
    medJTamp = zeros(1,nbPRbas);
    medJT2amp = zeros(1,nbPRbas);
    medPRamp = zeros(1,nbPRbas);
    for kk=1:nbPRbas
        medJTamp(kk) = nanmedian(bp_ecg_select_res(position.QRSoff(indPRbas(kk)):position.Ton(indPRbas(kk)))); % median amplitude QRSoff->Ton ('J point') = ~ST
        medJT2amp(kk) = nanmedian(bp_ecg_select_res(position.QRSoff(indPRbas(kk))+round(0.060*heasig.freq):position.Ton(indPRbas(kk)))); % median amplitude QRSoff+60 ms ('J point')->Ton = ~ST
        medPRamp(kk) = nanmedian(bp_ecg_select_res(position.Poff(indPRbas(kk)):position.QRSon(indPRbas(kk)))); % Poff to QRSon - median PR interval amplitude
    end
    medSTvar1 = median(medJTamp) - median(medPRamp); % J point
    medSTvar2 = nanmedian(medJT2amp) - median(medPRamp); % J point + 60 ms, nanmedian for JT2amp because of the + 60 ms which might end up being after the Ton sometime
    stdSTvar1 = nanstd(medJTamp - medPRamp);
    stdSTvar2 = nanstd(medJT2amp - medPRamp);
    
        % == NEW: checking how many time we get all the fiducials
    indALL = find(~isnan(position.Poff) & ~isnan(position.Pon) &... 
        ~isnan(position.QRSon) & ~isnan(position.QRSoff) & ...
        ~isnan(position.Toff) & ~isnan(position.Ton));
    nbALLwaves = length(indALL)/length(QRS); % proportion of cycles for which we manage to make a full segmentation
else
    disp('Not able to extract morphological features');
    medQT = NaN;
    medQT_b = NaN;
    medQT_fre = NaN;
    medQT_fra = NaN;
    medQT_hod = NaN;
    medP = NaN;
    medPR = NaN;
    stdPR = NaN;
    medT = NaN;
    medTamp = NaN;
    stdTamp = NaN;
    medTtype = NaN;
    stdP = NaN;
    stdT = NaN;
    nbpwaves = NaN;
    medPamp = NaN;
    stdPamp = NaN;
    medSTvar1 = NaN;
    medSTvar2 = NaN;
    medST = NaN;
    stdSTvar1 = NaN;
    stdSTvar2 = NaN;
    nbALLwaves = NaN;
    medPRseg = NaN;
end

%% END NEW
%%



X = [F1 meanSQI medianSQI quartileSQI length_clean...
    dist_jqrs_gqrs ...
    CosEn AFE OrC IrE PACe  ...
    sum(sg==-1)./length(sg) sum(amp_var>opt.bimod_delta)./length(sg) binar...
    min_rr max_rr median_rr nb_outliers ...
    out...
    QS QS_std R_amp_m R_amp_std...
    ratio max_freq...
    medQT medQT_b medQT_fre medQT_fra medQT_hod medP medPR stdPR ...
    medT medTamp stdTamp medTtype stdP stdT ...
    hrv_td.AVNN hrv_td.SDNN hrv_td.RMSSD hrv_td.pNN50 hrv_td.SEM ...
    hrv_frag.PIP hrv_frag.IALS hrv_frag.PSS hrv_frag.PAS...
    nbpwaves, ...
    medPamp, stdPamp, med_tb, medSTvar1, medSTvar2, medST, medPRseg];

%X(isnan(X)) = 0; % FIXME: added

% % == saving features matrix
% if exist('F_VEC.mat', 'file') == 2
%     load('F_VEC');
%     F = [F; X stdSTvar1 stdSTvar2 nbALLwaves];
% else
%     F = [X stdSTvar1 stdSTvar2 nbALLwaves];
% end
% save('F_VEC','F');


applyClassification;

% %%Normalise
% load minAndScale.mat
% test_data = bsxfun(@minus, X, tr_min);
% test_data = bsxfun(@rdivide, test_data, tr_scale);
% test_data = 2*(test_data-0.5); %[-1,1]
% 
% load modelSVMNormalAllMorpho.mat;
% [plop] = svmpredict(0,test_data,  model, '-q -b 0');
% 
% if plop==0
%     classifyResult = 'N';
% else
%     load modelSVMAFAllMorpho.mat;
%     [plop] = svmpredict(0,test_data,  model, '-q -b 0');
%     if plop==0
%         classifyResult = 'A';
%     else
%         load modelSVMNoiseAllMorpho.mat;
%         [plop] = svmpredict(0,test_data,  model, '-q -b 0');
%         if plop==0
%             classifyResult = '~';
%         else
%             classifyResult = 'O';
%         end
%     end
% end



