function [lp_outliers,newlp_outliers,newrr_lp,newQRS] = detect_outlier(QRS,fs,opt,sqi)
% detecting the outliers


% Copyright (C) 2017 Joachim A. Behar & Julien Oster
% Bio-electric and Bio-energetic system Group, Technion-IIT, Israel
% joachim.a.behar@gmail.com
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 2 of the License, or (at your
% option) any later version.
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.

rr = diff(QRS)/fs;
lp_outliers = [];
newlp_outliers = [];
rr_lp = rr;
newQRS=QRS;
opt.MIN_NB_FILT = 4; % FIXME: test Joachim

if length(rr)>opt.MIN_NB_FILT
    try
        %b_fiir = 1/(2 * opt.WIND_NB_SAMP).*ones(opt.WIND_NB_SAMP*2,1);
        %rr_lp = filtfilt(b_fiir,1,rr); % moving average window
        %try median instead...
        %rr_lp = medfilt1(rr,opt.WIND_NB_SAMP);
        rr_lp = medfilt1([rr(1)*ones(1,floor(opt.WIND_NB_SAMP./2)) rr rr(end)*ones(1,floor(opt.WIND_NB_SAMP./2))],opt.WIND_NB_SAMP);
        rr_lp = rr_lp(floor(opt.WIND_NB_SAMP./2)+1:end-floor(opt.WIND_NB_SAMP./2));
        lp_outliers = find(abs(rr-rr_lp)>(opt.WIND_TOL/100).*rr_lp); % find the outliers
    catch
        lp_outliers = [];
    end
else
    %    % = added to account for short segment with less than 10 beats
    %    % this resulted in decreasing the score- NEED TO INVESTIGATE
    %    lp_outliers = abs(diff(rr)./rr(1:end-1))>0.2;
end

newrr_lp = rr_lp;

%test if outliers are artefactuals peaks or actual ectopic beats
if ~isempty(lp_outliers)
    x1 = size(lp_outliers);
    x2 = size(lp_outliers);
    y1 = size(lp_outliers);
    y2 = size(lp_outliers);
    
    for i_outlier=1:length(lp_outliers)
        i1 = lp_outliers(i_outlier)-1;
        i2 = lp_outliers(i_outlier);
        i3 = lp_outliers(i_outlier)+1;
        
        if i1>0
            x1(i_outlier) = (rr(i1)-rr_lp(i1))./rr_lp(i1);
            y1(i_outlier) = (rr(i2)-rr_lp(i2))./rr_lp(i2);
        end
        
        if i3<=length(rr)
            x2(i_outlier)  = y1(i_outlier);
            y2(i_outlier)  = (rr(i3)-rr_lp(i3))./rr_lp(i3);
        end
    end
    %detect which RR are outliers.. depends if couple (x1,y1) or (x2,y2) are
    %in low and left quadran of Poincare plot
    falseOutlier = lp_outliers((x1<opt.False_Outlier_Thres&y1<opt.False_Outlier_Thres))-1;
    falseOutlierBis = lp_outliers((x2<opt.False_Outlier_Thres&y2<opt.False_Outlier_Thres));
    FalseOutliers = sort(unique([falseOutlier falseOutlierBis]));
    
    %now there is the need to suppress artefactual QRS
    if ~isempty(FalseOutliers)
        newInd =  ones(size(QRS));
        newInd(FalseOutliers+1)=0;
        newQRS = QRS(find(newInd));
        
        newrr = diff(newQRS)/fs;
        new_lp_outliers = [];
        if length(newrr)>opt.MIN_NB_FILT
            try
                %b_fiir = 1/(2 * opt.WIND_NB_SAMP).*ones(opt.WIND_NB_SAMP*2,1);
                %rr_lp = filtfilt(b_fiir,1,rr); % moving average window
                %try median instead...
                %rr_lp = medfilt1(rr,opt.WIND_NB_SAMP);
                newrr_lp = medfilt1([newrr(1)*ones(1,floor(opt.WIND_NB_SAMP./2)) newrr newrr(end)*ones(1,floor(opt.WIND_NB_SAMP./2))],opt.WIND_NB_SAMP);
                newrr_lp = newrr_lp(floor(opt.WIND_NB_SAMP./2)+1:end-floor(opt.WIND_NB_SAMP./2));
                newlp_outliers = find(abs(newrr-newrr_lp)>(opt.WIND_TOL/100).*newrr_lp); % find the outliers
            catch
                newlp_outliers = [];
            end
        else
            %    % = added to account for short segment with less than 10 beats
            %    % this resulted in decreasing the score- NEED TO INVESTIGATE
            %    lp_outliers = abs(diff(rr)./rr(1:end-1))>0.2;
        end
    else
        newlp_outliers = lp_outliers;
        newQRS = QRS;
    end
    
end


% % %% == FIXME: test
% % perhaps try to lower the agreement threshold for this second step to
% % 0.1 or similar
% % lp.WIND_TOL = 10;
% QRS_temp = QRS;
% QRS_temp(lp_outliers+1)=[]; % removing potential extra peaks
% rr = diff(QRS_temp)/fs;
% if length(rr)>opt.MIN_NB_FILT
%     try
%         b_fiir = 1/(2 * opt.WIND_NB_SAMP).*ones(opt.WIND_NB_SAMP*2,1);
%         rr_lp = filtfilt(b_fiir,1,rr); % moving average window
%         lp_outliers = find(abs(rr-rr_lp)>(opt.WIND_TOL/100).*rr_lp); % find the outliers
%     catch
%         lp_outliers = [];
%     end
%     QRS = QRS_temp;
% end
%

% ==========================================
% removing the outliers detected on the part where the sqi is not good
qrs_out = QRS;
sqi_ect = round(qrs_out(lp_outliers+1)/fs);
sqi_ect(sqi_ect==0)=1;
sqi_out = sqi(sqi_ect);
ind_sqi_out = find(sqi_out<opt.sqi_outliers);
lp_outliers(ind_sqi_out)=[];
%new outliers as well...
if ~isempty(newlp_outliers)
sqi_ect = round(newQRS(newlp_outliers+1)/fs);
sqi_ect(sqi_ect==0)=1;
sqi_out = sqi(sqi_ect);
ind_sqi_out = find(sqi_out<opt.sqi_outliers);
newlp_outliers(ind_sqi_out)=[];
end
if ~isempty(ind_sqi_out); disp('removed some false ecto?'); end;

end