function [ sqi, tsqi ] = ecgsqi( ann1, ann2, THR, SIZE_WIND, REG_WIN, LG_MED, LG_REC, N_WIN )

% Challenge 2014: Alistair Johnson
% Challenge 2017: modified, joachim behar, 27-03-2017
% NOTE: the first value of the sqi is computed for t=0 seconds

% Copyright (C) 2017 Joachim A. Behar
% Bio-electric and Bio-energetic system Group, Technion-IIT, Israel
% joachim.a.behar@gmail.com
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 2 of the License, or (at your
% option) any later version.
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.

if nargin < 7
    error('ecgsqi:notEnoughArguments',...
        'ecgsqi.m received %d arguments, expected 7.',...
        nargin)
end

xi1 = (0:REG_WIN:LG_REC)';

for w=1:numel(xi1)
    idx1_sel = ann1>w-1-floor(15/2) & ann1<w-1+ceil(15/2); % FIXME: TESTING adjusting window size with median heart rate
    fac = 70/median(60./diff(ann1(idx1_sel))); % FIXME: TEST. Need to store fac and return it as output of the function
    
    %== FIXME: this code was changed by Joachim, 27-03-2017
    idx1 = ann1>w-1-floor(fac*SIZE_WIND/2) & ann1<w-1+ceil(fac*SIZE_WIND/2); % FIXME: added -1
    idx2 = ann2>w-1-floor(fac*SIZE_WIND/2) & ann2<w-1+ceil(fac*SIZE_WIND/2); % FIXME: added -1
    
    %====> added Joachim, re-defininf F1
    refqrs = ann1(idx1);
    testqrs = ann2(idx2);
    fs = 300;
    [F1(w),~] = bsqi_matlab( refqrs*fs, testqrs*fs,THR,fs);
end

% NaN values indicate no ECG data for either lead - definitely bad quality
F1(isnan(F1)) = 0;

% Remove the non-relevant segments
idxRem = xi1 >= LG_REC+1; %+1 because the first value of the sqi is for time=0 sec
F1(idxRem) = [];
xi1(idxRem) = [];

%% Now smooth the SQI (F1)
F1=F1';
if size(F1,1) < (LG_MED*2+1)
    F1smooth = F1;
else
    F1smooth = nan(size(F1,1),2*LG_MED+1);
    for k=1:LG_MED
        % create a lagged version of F1
        F1smooth(:,k) = vertcat(ones(k,1),F1(1:end-k));
        
        % create a led version of F1
        F1smooth(:,k+LG_MED) = vertcat(F1(k+1:end),ones(k,1));
    end
    F1smooth(:,end) = F1;
    % take the min in a window of 3 samples around the current F1 score
    F1smooth = median(F1smooth'); % changed JB
end
sqi = F1smooth;
tsqi = xi1;

end
