function [ratio, max_freq] = f_wave_detection(ecg,qrs,fs)
% this function compute the dominant frequency mode in the badn 4-45 Hz
% after subtraction of the ECG cycle.
%
% input
%   ecg: ecg time series
%   qrs: R peak location
%   fs: sampling frequency
%
% ouput
%   ratio: ratio of power in the band 5-9 Hz versus total power
%   max_freq: dominant frequency
%
% Copyright (C) 2017 Joachim A. Behar
% Bio-electric and Bio-energetic system Group, Technion-IIT, Israel
% joachim.a.behar@gmail.com
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 2 of the License, or (at your
% option) any later version.
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.

try
    % == prefiltering
    LOW_CUT_FREQ = 4;
    HIGH_CUT_FREQ = 45;
    [b_lp,a_lp] = butter(5,HIGH_CUT_FREQ/(fs/2),'high');
    [b_bas,a_bas] = butter(2,LOW_CUT_FREQ/(fs/2),'high');
    ecg = ecg-mean(ecg); % (1) centre
    bpfecg = ecg'-filtfilt(b_lp,a_lp,ecg'); % (2) remove higher freq (zero phase)
    bpfecg = filtfilt(b_bas,a_bas,bpfecg); % (3) remove baseline (zero phase)

    % == remove QRS complex
    residual = mecg_cancellation(qrs,bpfecg,'TS-PCA',5,2,fs);

    % == spectral analysis
    [Pxx,F,~] = pwelch(residual,[],[],[],fs);
    [~,ind] = max(Pxx);
    max_freq = F(ind);
    ratio = sum(Pxx((F>5 & F<9)))/sum(Pxx);
    
catch
    max_freq = NaN;
    ratio = NaN;
end

end