function [F1,sqi,tsqi,QRS,start_seg,end_seg,meanSQI,medianSQI,quartileSQI,dist_jqrs_gqrs] = sqi_step(jqrs,epltd,anns_gqrs_adj,fs,NB_PT,opt)
%
% Copyright (C) 2017 Joachim A. Behar
% Bio-electric and Bio-energetic system Group, Technion-IIT, Israel
% joachim.a.behar@gmail.com
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 2 of the License, or (at your
% option) any later version.
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.

DEFINE_SELECT_SEG = 1; % enable or disable subsegment selection based on SQI

[F1_gqrs_jqrs,~,dist_jqrs_gqrs] = bsqi_matlab(anns_gqrs_adj,jqrs',opt.WIND_SQI,fs);
[F1_gqrs_epltd,~] = bsqi_matlab(anns_gqrs_adj,epltd,opt.WIND_SQI,fs);
[F1_epltd_jqrs,~] = bsqi_matlab(epltd,jqrs',opt.WIND_SQI,fs);

[F1,ind] = max([F1_gqrs_jqrs F1_gqrs_epltd F1_epltd_jqrs]);

% == Computing second by second SQI for selecting subsegment
LG_REC = NB_PT/fs; % length of the record in seconds
N_WIN = ceil(LG_REC/opt.REG_WIN);


try
    if ind==1
        %QRS = jqrs;
        [sqi,tsqi] = ecgsqi(anns_gqrs_adj/fs,jqrs'/fs,opt.WIND_SQI,opt.SIZE_WIND,opt.REG_WIN,opt.LG_MED,LG_REC,N_WIN);
    elseif ind==2
        %QRS = anns_gqrs_adj;
        [sqi,tsqi] = ecgsqi(anns_gqrs_adj/fs,epltd/fs,opt.WIND_SQI,opt.SIZE_WIND,opt.REG_WIN,opt.LG_MED,LG_REC,N_WIN);
    elseif ind==3
        %QRS = jqrs;
        [sqi,tsqi] = ecgsqi(epltd/fs,jqrs'/fs,opt.WIND_SQI,opt.SIZE_WIND,opt.REG_WIN,opt.LG_MED,LG_REC,N_WIN);
    end
catch
    disp('entering catch');
    tsqi = 0:opt.REG_WIN:LG_REC;
    sqi = zeros(1,length(tsqi));
end

QRS = jqrs; % always using jqrs as base. This showed to improve the results
    
if DEFINE_SELECT_SEG
    sqi_thres = sqi>opt.THRES_SQI;
    is=find(diff([0 sqi_thres])==1);
    ie=find(diff([sqi_thres 0])==-1);
    lgt = ie-is+1;
    [~,ind_lgt] = max(lgt);
    start_seg = is(ind_lgt)-1;
    end_seg = ie(ind_lgt)-1;
    if end_seg>LG_REC; end_seg=floor(LG_REC); end;

    if end_seg-start_seg>opt.MIN_SUB_SEG_TIME
        % if we have a minimum of MIN_SUB_SEG_TIME seconds of good quality then we select
        % this specific segment and label the record is classified as non-noisy
        % this is to account for segments where part is noise and part is
        % usable data

        QRS = QRS(QRS<end_seg*fs & QRS>start_seg*fs); % only keep the corresponding segment
        %test JOs
        if start_seg==0
            %if mean(sqi(1:end_seg)>0.9) %%FIXME big issue with test... and
            %parenthesis
                meanSQI = mean(sqi(1:end_seg));%2;
                medianSQI = median(sqi(1:end_seg));%2;
                plopSQI = sort(sqi(1:end_seg));
                quartileSQI = plopSQI(round(length(plopSQI)./4));
            %end
        else
            %if mean(sqi(is(ind_lgt):ie(ind_lgt))>0.9) %%FIXME big issue with test...
                meanSQI = mean(sqi(is(ind_lgt):ie(ind_lgt)));%2;
                medianSQI = median(sqi(is(ind_lgt):ie(ind_lgt)));%2;
                plopSQI = sort(sqi(is(ind_lgt):ie(ind_lgt)));
                quartileSQI = plopSQI(round(length(plopSQI)./4));
            %end
        end
    else
        % ADDED 21-08-2017
        start_seg = 0;
        end_seg = floor(LG_REC);
        %------
        meanSQI = mean(sqi);%2;
        medianSQI = median(sqi);%2;
        plopSQI = sort(sqi);
        quartileSQI = plopSQI(round(length(plopSQI)./4));
    end

    if size(QRS,1)>size(QRS,2); QRS = QRS'; end;

else
    start_seg = 0;
    end_seg = floor(LG_REC);
end

end